;;
;; mailinglist.scm
;;
;; Common high-level mailinglist management procedures.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit mailinglist))

(module
 mailinglist
 (
  check-mailing-list
  print-mailing-list-checks

  rada-ml-pred?
  )

 (import scheme
	 (chicken base)
	 (chicken format)
	 racket-kwargs
	 mailman
	 mbase
	 util-string
	 brmember)

 (define* (check-mailing-list MB mls name #:pred? (pred? #f) #:suspended (suspended #f))
   (define ml (find-mailman-list mls name))
   (when ml
     (let-values (((missing surplus)
		   (mailman-compare-members ml
					    (mbase-active-emails MB
								 #:suspended suspended
								 #:pred? pred?
								 ))))
       (if (null? (cdr ml))
	   (print "Skipping ML check - not loaded")
	   (if (and (null? missing)
		    (null? surplus))
	       (print (format "~a mailing list membership in sync." (string-capitalize name)))
	       (let ()
		 (print (format "~a mailing list:" (string-capitalize name)))
		 (when (not (null? missing))
		   (print "  Missing: " missing))
		 (when (not (null? surplus))
		   (print "  Outsiders: " surplus))))))))

 (define (print-mailing-list-checks MB MLS)
   (check-mailing-list MB MLS "internal" #:suspended #t)
   (check-mailing-list MB MLS "rada"
		       #:pred? rada-ml-pred?)
   (check-mailing-list MB MLS "rk" #:pred? brmember-revision?))

 (define (rada-ml-pred? mr)
   (or (brmember-council? mr)
       (brmember-chair? mr)
       (brmember-revision? mr)))

 )
